==> xray_audit-1.5.1.tar.gz: xray_audit/src/Plugin/xray_audit/tasks/PackageReport/XrayAuditModulesPlugin.php <==
<?php

namespace Drupal\xray_audit\Plugin\xray_audit\tasks\PackageReport;

use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Drupal\xray_audit\Services\CsvDownloadManagerInterface;
use Drupal\xray_audit\Services\PluginRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of modules status.
 *
 * @XrayAuditTaskPlugin (
 *   id = "modules",
 *   label = @Translation("Modules Status (sync config files)"),
 *   description = @Translation("Module status against sync configuration files (include config splits)."),
 *   group = "package",
 *   sort = 1,
 *   operations = {
 *      "modules" = {
 *          "label" = "Drupal Modules",
 *          "description" = "Drupal Modules."
 *       }
 *    },
 * )
 */
class XrayAuditModulesPlugin extends XrayAuditTaskPluginBase {

  /**
   * Service "config.storage.sync".
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $configStoreSync;

  /**
   * Service "extension.list.module".
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $extensionListModule;

  /**
   * Service "xray_audit.plugin_repository".
   *
   * @var \Drupal\xray_audit\Services\PluginRepositoryInterface
   */
  protected $pluginRepository;

  /**
   * Service "xray_audit.csv_download_manager".
   *
   * @var \Drupal\xray_audit\Services\CsvDownloadManagerInterface
   */
  protected $csvDownloadManager;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\StorageInterface $configStoreSync
   *   Service "config.storage.sync".
   * @param \Drupal\Core\Extension\ModuleExtensionList $extensionListModule
   *   Service "extension.list.module".
   * @param \Drupal\xray_audit\Services\PluginRepositoryInterface $pluginRepository
   *   Service "exray_audit.plugin_repository".
   * @param \Drupal\xray_audit\Services\CsvDownloadManagerInterface $csvDownloadManager
   *   Service "xray_audit.csv_download_manager".
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition, StorageInterface $configStoreSync, ModuleExtensionList $extensionListModule, PluginRepositoryInterface $pluginRepository, CsvDownloadManagerInterface $csvDownloadManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configStoreSync = $configStoreSync;
    $this->extensionListModule = $extensionListModule;
    $this->pluginRepository = $pluginRepository;
    $this->csvDownloadManager = $csvDownloadManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.storage.sync'),
      $container->get('extension.list.module'),
      $container->get('xray_audit.plugin_repository'),
      $container->get('xray_audit.csv_download_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {

      case 'modules':
        return $this->getModuleStatus();

    }
    return [];

  }

  /**
   * Modules custom and contrib.
   *
   * @return array
   *   Data.
   */
  protected function getModuleStatus() {
    $resultTable = [];
    $list_modules_instaled = $this->extensionListModule->getList();
    $modules_actives = $this->getModulesActivesByConfigSync();
    if (empty($list_modules_instaled) || empty($modules_actives)) {
      return $resultTable;
    }

    // Build the data.
    foreach ($list_modules_instaled as $machine_name => $module) {
      $group = $module->origin === 'core' ? 'core' : 'not_core';
      $info = $module->info;
      $resultTable[$group][$machine_name] = [
        'package' => $info['package'] ?? '',
        'project' => $info['project'] ?? '',
        'module' => $info['name'],
        'machine_name' => "$machine_name",
        'used' => isset($modules_actives[$machine_name]) ? $this->t('Yes') : $this->t('No'),
        'enabled_by' => $modules_actives[$machine_name] ?? '',
        'version' => $info['version'] ?? '-',
        'group' => $group,
      ];
    }

    return $resultTable;

  }

  /**
   * {@inheritDoc}
   */
  public function buildDataRenderArray(array $data, string $operation = NULL) {
    if (empty($data)) {
      return [
        "#markup" => $this->t("Not data found,
          maybe the configuration files have not been exported."),
      ];
    }

    $headerTable = [
      $this->t('Package'),
      $this->t('Project'),
      $this->t('Module'),
      $this->t('Machine name'),
      $this->t('Used'),
      $this->t('Activated by'),
      $this->t('Version'),
    ];

    $group_info['core'] = [
      'title' => $this->t("Modules core"),
      'description' => $this->t("Modules belong to the core"),
    ];
    $group_info['not_core'] = [
      'title' => $this->t("Modules custom and contrib"),
      'description' => $this->t("Modules custom and contrib"),
    ];

    $builds = [];
    foreach ($data as $type => $group_modules) {
      // Sort the modules by package.
      $key_values = array_column($group_modules, 'package');
      array_multisort($key_values, SORT_ASC, $group_modules);

      $builds[$type] = [
        '#theme' => 'details',
        '#title' => $group_info[$type]['title'],
        '#description' => $group_info[$type]['description'],
        '#attributes' => ['class' => ['package-listing']],
        '#summary_attributes' => [],
        '#children' => [
          'tables' => [
            '#theme' => 'table',
            '#header' => $headerTable,
            '#rows' => $group_modules,
          ],
        ],
      ];
    }
    $builds['download'] = [
      '#type' => 'link',
      '#url' => $this->pluginRepository->getTaskPageOperationUrl('package', 'modules', 'modules', ['download']),
      '#title' => $this->t('Download'),
      '#attributes' => [
        'class' => [
          'button',
          'button--primary',
          'button--small',
        ],
      ],
    ];
    if ($this->csvDownloadManager->downloadCsv($operation)) {

      $headers = [
        'Package',
        'Project',
        'Module',
        'Machine name',
        'Used',
        'Activated by',
        'Version',
        'Core or custom and contrib',
      ];

      $csvData = $this->getAlldataAtOnce($operation);
      $this->csvDownloadManager->createCsv($csvData, $headers, $operation);
    }

    return $builds;

  }

  /**
   * Get the modules actives by configuration files.
   *
   * @return array
   *   Status modules.
   */
  protected function getModulesActivesByConfigSync(): array {
    // Set the module actives checking the configuration files.
    $modules_actives = [];
    // Get the name of all the config splits configuration files.
    $all_config_splits = $this->configStoreSync->listAll('config_split.config_split.');

    // Add the core.extension (modules enable by default).
    $all_config_splits[] = 'core.extension';

    // Storage the active modules.
    foreach ($all_config_splits as $config_split_name) {
      $config_split = $this->configStoreSync->read($config_split_name);
      if (empty($config_split['module'])) {
        continue;
      }
      foreach ($config_split['module'] as $name => $value) {
        if (!isset($modules_actives[$name])) {
          $modules_actives[$name] = '';
        }
        else {
          $modules_actives[$name] .= ', ';
        }
        $modules_actives[$name] .= $config_split_name;
      }
    }

    return $modules_actives;

  }

  /**
   * Returns all module's data in one array.
   *
   * @param string|null $operation
   *   Operation.
   *
   * @return array
   *   All data from operation.
   */
  public function getAlldataAtOnce(?string $operation): array {
    $data_per_group = $this->getDataOperationResult($operation);
    $data = [];

    foreach ($data_per_group as $values) {
      foreach ($values as $value) {
        $data[] = $value;
      }
    }
    return $data;
  }

}

==> xray_audit-1.5.3.tar.gz: xray_audit/src/Plugin/xray_audit/tasks/PackageReport/XrayAuditModulesPlugin.php <==
<?php

namespace Drupal\xray_audit\Plugin\xray_audit\tasks\PackageReport;

use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Drupal\xray_audit\Services\CsvDownloadManagerInterface;
use Drupal\xray_audit\Services\PluginRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of modules status.
 *
 * @XrayAuditTaskPlugin (
 *   id = "modules",
 *   label = @Translation("Modules Status (sync config files)"),
 *   description = @Translation("Module status against sync configuration files (include config splits)."),
 *   group = "package",
 *   sort = 1,
 *   operations = {
 *      "modules" = {
 *          "label" = "Drupal Modules",
 *          "description" = "Drupal Modules."
 *       }
 *    },
 * )
 */
class XrayAuditModulesPlugin extends XrayAuditTaskPluginBase {

  /**
   * Service "config.storage.sync".
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $configStoreSync;

  /**
   * Service "extension.list.module".
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $extensionListModule;

  /**
   * Service "xray_audit.plugin_repository".
   *
   * @var \Drupal\xray_audit\Services\PluginRepositoryInterface
   */
  protected $pluginRepository;

  /**
   * Service "xray_audit.csv_download_manager".
   *
   * @var \Drupal\xray_audit\Services\CsvDownloadManagerInterface
   */
  protected $csvDownloadManager;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\StorageInterface $configStoreSync
   *   Service "config.storage.sync".
   * @param \Drupal\Core\Extension\ModuleExtensionList $extensionListModule
   *   Service "extension.list.module".
   * @param \Drupal\xray_audit\Services\PluginRepositoryInterface $pluginRepository
   *   Service "xray_audit.plugin_repository".
   * @param \Drupal\xray_audit\Services\CsvDownloadManagerInterface $csvDownloadManager
   *   Service "xray_audit.csv_download_manager".
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module handler.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    StorageInterface $configStoreSync,
    ModuleExtensionList $extensionListModule,
    PluginRepositoryInterface $pluginRepository,
    CsvDownloadManagerInterface $csvDownloadManager,
    ModuleHandlerInterface $module_handler,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configStoreSync = $configStoreSync;
    $this->extensionListModule = $extensionListModule;
    $this->pluginRepository = $pluginRepository;
    $this->csvDownloadManager = $csvDownloadManager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.storage.sync'),
      $container->get('extension.list.module'),
      $container->get('xray_audit.plugin_repository'),
      $container->get('xray_audit.csv_download_manager'),
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = NULL) {
    switch ($operation) {

      case 'modules':
        return $this->getModuleStatus();

    }
    return [];

  }

  /**
   * Modules custom and contrib.
   *
   * @return array
   *   Data.
   */
  protected function getModuleStatus() {
    $resultTable = [];
    $list_modules_installed = $this->extensionListModule->getList();
    $modules_actives = $this->getModulesActivesByConfigSync();
    if (empty($list_modules_installed) || empty($modules_actives)) {
      return $resultTable;
    }

    $projects_update_info = $this->getProjectsUpdateInfo();

    // Build the data.
    foreach ($list_modules_installed as $machine_name => $module) {
      $group = $this->determineModuleGroup($module);

      $info = $module->info;
      $resultTable[$group][$machine_name] = [
        'package' => $info['package'] ?? '',
        'project' => $info['project'] ?? '',
        'module' => $info['name'],
        'machine_name' => "$machine_name",
        'description' => $info['description'] ?? '',
        'used' => isset($modules_actives[$machine_name]) ? $this->t('Yes') : $this->t('No'),
        'enabled_by' => $modules_actives[$machine_name] ?? '',
        'version' => $info['version'] ?? '-',
        'recommended_version' => $projects_update_info[$machine_name]['recommended'] ?? '',
        'group' => $group,
      ];
    }

    return $resultTable;

  }

  /**
   * {@inheritDoc}
   *
   * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
   */
  public function buildDataRenderArray(array $data, string $operation = NULL) {
    $group_info = [];

    if (empty($data)) {
      return [
        "#markup" => $this->t("Not data found,
          maybe the configuration files have not been exported."),
      ];
    }

    $headerTable = [
      $this->t('Package'),
      $this->t('Project'),
      $this->t('Module'),
      $this->t('Machine name'),
      $this->t('Description'),
      $this->t('Used'),
      $this->t('Activated by'),
      $this->t('Version'),
      $this->t('Recommended version'),
    ];

    $group_info['core'] = [
      'title' => $this->t("Modules core"),
      'description' => $this->t("Modules belong to the core"),
    ];

    $group_info['contrib'] = [
      'title' => $this->t("Modules contrib"),
      'description' => $this->t("Modules contrib"),
    ];

    $group_info['custom'] = [
      'title' => $this->t("Modules custom"),
      'description' => $this->t("Modules custom"),
    ];

    $group_info['profile'] = [
      'title' => $this->t("Profiles"),
      'description' => $this->t("Profiles"),
    ];

    $group_info['other'] = [
      'title' => $this->t("Others"),
      'description' => $this->t("Others"),
    ];

    $builds = [];
    foreach ($data as $type => $group_modules) {
      // Sort the modules by package.
      $key_values = array_column($group_modules, 'package');
      array_multisort($key_values, SORT_ASC, $group_modules);

      $builds[$type] = [
        '#theme' => 'details',
        '#title' => $group_info[$type]['title'],
        '#description' => $group_info[$type]['description'],
        '#attributes' => ['class' => ['package-listing']],
        '#summary_attributes' => [],
        '#children' => [
          'tables' => [
            '#theme' => 'table',
            '#header' => $headerTable,
            '#rows' => $group_modules,
          ],
        ],
      ];
    }
    $builds['download'] = [
      '#type' => 'link',
      '#url' => $this->pluginRepository->getTaskPageOperationUrl('package', 'modules', 'modules', ['download']),
      '#title' => $this->t('Download'),
      '#attributes' => [
        'class' => [
          'button',
          'button--primary',
          'button--small',
        ],
      ],
    ];
    if ($this->csvDownloadManager->downloadCsv()) {

      $headers = [
        'Package',
        'Project',
        'Module',
        'Machine name',
        'Description',
        'Used',
        'Activated by',
        'Version',
        'Recommended version',
        'Core, contrib, custom or other',
      ];

      $csvData = $this->getAllDataAtOnce($operation);
      $operation = $operation ?? '';
      $this->csvDownloadManager->createCsv($csvData, $headers, $operation);
    }

    return $builds;

  }

  /**
   * Get the modules actives by configuration files.
   *
   * @return array
   *   Status modules.
   */
  protected function getModulesActivesByConfigSync(): array {
    // Set the module actives checking the configuration files.
    $modules_actives = [];
    // Get the name of all the config splits configuration files.
    $all_config_splits = $this->configStoreSync->listAll('config_split.config_split.');

    // Add the core.extension (modules enable by default).
    $all_config_splits[] = 'core.extension';

    // Storage the active modules.
    foreach ($all_config_splits as $config_split_name) {
      $config_split = $this->configStoreSync->read($config_split_name);
      if (empty($config_split['module'])) {
        continue;
      }
      $module_names = array_keys($config_split['module']);
      foreach ($module_names as $name) {
        if (!isset($modules_actives[$name])) {
          $modules_actives[$name] = '';
        }
        else {
          $modules_actives[$name] .= ', ';
        }
        $modules_actives[$name] .= $config_split_name;
      }
    }

    return $modules_actives;

  }

  /**
   * Determines to which group the module belongs.
   *
   * @param \Drupal\Core\Extension\Extension $module
   *   Module.
   *
   * @return string
   *   Module group.
   */
  protected function determineModuleGroup(Extension $module): string {
    $group = '';
    $module_path = $module->getPath();
    $module_info = $module->info;

    switch (TRUE) {
      case (isset($module->origin) && $module->origin === 'core'):
        $group = 'core';
        break;

      case strpos($module_path, 'modules/contrib') !== FALSE:
        $group = 'contrib';
        break;

      case strpos($module_path, 'modules/custom') !== FALSE:
        $group = 'custom';
        break;

      case strpos($module_path, 'profiles/') !== FALSE:
        $group = 'profile';
        break;

      // At last resource to check if contrib or not, check info added by
      // drupal.
      case !empty($module_info['project']):
        $group = 'contrib';
        break;

      // And if none of the conditions before were met and the module
      // is inside anywhere in modules just consider it custom.
      case strpos($module_path, 'modules/') !== FALSE:
        $group = 'custom';
        break;

      default:
        $group = 'other';
        break;
    }

    return $group;
  }

  /**
   * Get projects update info.
   *
   * @return array
   *   Projects update info.
   */
  protected function getProjectsUpdateInfo(): array {
    $projects_update_info = [];

    if ($this->moduleHandler->moduleExists('update')) {
      $available = update_get_available(TRUE);

      if (!empty($available) && function_exists('update_calculate_project_data')) {
        $projects_update_info = update_calculate_project_data($available);
      }
    }

    return $projects_update_info;
  }

  /**
   * Returns all module's data in one array.
   *
   * @param string|null $operation
   *   Operation.
   *
   * @return array
   *   All data from operation.
   */
  public function getAllDataAtOnce(?string $operation): array {
    $data_per_group = $this->getDataOperationResult($operation);
    $data = [];

    foreach ($data_per_group as $values) {
      foreach ($values as $value) {
        $data[] = $value;
      }
    }
    return $data;
  }

}
